﻿using System;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
using TrainCrew;
using System.Drawing;

namespace TSMasconInput
{
    public partial class Form1 : Form
    {
        SerialPort port;
        Timer tim = new Timer();
        string lastStr = "";
        InputAction W_Action = InputAction.HornEle;
        InputAction G_Action = InputAction.Buzzer;
        InputAction Y_Action = InputAction.GradientStart;
        InputAction R_Action = InputAction.HornAir;

        public Form1()
        {
            InitializeComponent();

            //初期化。 TrainCrewInput.dllの初期化はこの関数のみで大丈夫です
            TrainCrewInput.Init();


            //ボタン関係
            comboBox2.Text = "電笛";
            comboBox4.Text = "ブザー";
            comboBox3.Text = "勾配起動";
            comboBox5.Text = "空笛";

            //ポート等の設定
            var names = SerialPort.GetPortNames();
            foreach (var nm in names)
            {
                comboBox1.Items.Add(nm);
                if(comboBox1.Text == "") comboBox1.Text = nm;
            }

            btn_close.Enabled = false;

            FormClosed += Form1_FormClosed;

            tim.Tick += Tim_Tick;
            tim.Interval = 60;
            tim.Start();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            Button b = (Button)sender;
            switch (b.Text)
            {
                case "接続":
                    try
                    {
                        string portName = comboBox1.Text;

                        port = new SerialPort(portName, 19200);

                        port.DtrEnable = true;
                        port.RtsEnable = true;
                        port.DataBits = 8;
                        port.NewLine = "\r";
                        port.Parity = Parity.Even;
                        port.Open();
                        
                        btn_open.Enabled = false;
                        btn_close.Enabled = true;

                        SetNotch(-8);
                    }
                    catch (Exception ex)
                    {
                        System.Windows.Forms.MessageBox.Show(ex.ToString());
                    }
                    break;
                case "切断":
                    port.Close();
                    port.Dispose();
                    port = null;
                    btn_open.Enabled = true;
                    btn_close.Enabled = false;
                    break;
            }
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (port != null && port.IsOpen)
            {
                port.Close();
                port = null;
            }

            //終了処理
            TrainCrewInput.Dispose();
        }

        StringBuilder sb = new StringBuilder();
        private void Tim_Tick(object sender, EventArgs e)
        {
            //TRAIN CREWへの入力
            if (port != null && port.BytesToRead != 0)
            {

                string Rtstext = "";
                try
                {
                    Rtstext = port.ReadExisting();
                }
                catch (Exception ex)
                {
                    Rtstext = "";
                    label7.Text = "Error:" + ex.Message;
                    return;
                }

                if (!string.IsNullOrEmpty(Rtstext))
                {
                    string[] array = (this.lastStr + Rtstext).Split('\r');

                    for (int i = 0; i < array.Length - 1; i++)
                    {
                        string str = array[i];
                        switch (str)
                        {
                            //ノッチ
                            case "TSB20": SetNotch(-8); break; //EB
                            case "TSB30": SetNotch(-8); break; //EB
                            case "TSB40": SetNotch(-7); break; //B6
                            case "TSE99": SetNotch(-6); break; //B5
                            case "TSA05": SetNotch(-5); break; //B4
                            case "TSA15": SetNotch(-4); break; //B3
                            case "TSA25": SetNotch(-3); break; //B2
                            case "TSA35": SetNotch(-2); break; //B1
                            case "TSA45": SetNotch(-1); break; //抑速
                            case "TSA50": SetNotch(0); break; //N
                            case "TSA55": SetNotch(1); break; //P1
                            case "TSA65": SetNotch(2); break; //P2
                            case "TSA75": SetNotch(3); break; //P3
                            case "TSA85": SetNotch(4); break; //P4
                            case "TSA95": SetNotch(5); break; //P5

                            //逆転器
                            case "TSG99": TrainCrewInput.SetReverser(1); break;
                            case "TSG50": TrainCrewInput.SetReverser(0); break;
                            case "TSG00": TrainCrewInput.SetReverser(-1); break;

                            //白ボタン
                            case "TSK99": TrainCrewInput.SetButton(W_Action, true); break;
                            case "TSK00": TrainCrewInput.SetButton(W_Action, false); break;
                            
                            //緑ボタン
                            case "TSX99": TrainCrewInput.SetButton(G_Action, true); break;
                            case "TSX00": TrainCrewInput.SetButton(G_Action, false); break;
                            
                            //黄ボタン
                            case "TSY99": TrainCrewInput.SetButton(Y_Action, true); break;
                            case "TSY00": TrainCrewInput.SetButton(Y_Action, false); break;
                            
                            //赤ボタン
                            case "TSZ99": TrainCrewInput.SetButton(R_Action, true); break;
                            case "TSZ00": TrainCrewInput.SetButton(R_Action, false); break;
                        }
                    }

                    lastStr = array[array.Length - 1];
                }
            }



            //TRAIN CREWからの出力
            var state = TrainCrewInput.GetTrainState();

            sb.Clear();
            sb.AppendLine("速度：" + state.Speed.ToString("0.0") + "km/h");
            sb.AppendLine("戸閉：" + (state.AllClose ? "●" : "○"));
            sb.AppendLine("MR圧力：" + state.MR_Press.ToString("0") + "kPa");
            if (state.CarStates.Count > 0)
            {
                sb.AppendLine("BC圧力：" + state.CarStates[0].BC_Press.ToString("0") + "kPa");
            }
            label1.Text = sb.ToString();


            //表示灯
            sb.Clear();
            sb.AppendLine((state.Lamps[PanelLamp.DoorClose] ? "●" : "○") + "戸　閉");
            sb.AppendLine((state.Lamps[PanelLamp.ATS_Ready] ? "●" : "○") + "ATS正常");
            sb.AppendLine((state.Lamps[PanelLamp.ATS_BrakeApply] ? "●" : "○") + "ATS動作");
            sb.AppendLine((state.Lamps[PanelLamp.ATS_Open] ? "●" : "○") + "ATS開放");
            sb.AppendLine((state.Lamps[PanelLamp.RegenerativeBrake] ? "●" : "○") + "回　生");
            sb.AppendLine((state.Lamps[PanelLamp.EB_Timer] ? "●" : "○") + "E　B");
            sb.AppendLine((state.Lamps[PanelLamp.EmagencyBrake] ? "●" : "○") + "非常ﾌﾞﾚｰｷ");
            sb.AppendLine((state.Lamps[PanelLamp.Overload] ? "●" : "○") + "過負荷");

            labelPnl.Text = sb.ToString();


            //ATS表示器
            sb.Clear();
            sb.AppendLine(state.ATS_Class);
            bool show = true; //点滅用
            switch (state.ATS_State)
            {
                case "P接近":
                case "B動作":
                case "EB":
                    if(Environment.TickCount % 1000 < 500) show = false;
                    break;
            }
            sb.AppendLine(state.ATS_Speed);
            sb.AppendLine(show ? state.ATS_State : "");
            labelATS.Text = sb.ToString();

            //運転情報
            sb.Clear();
            sb.AppendLine(state.NowTime.ToString());
            sb.AppendLine("列車番号：" + state.diaName);
            sb.AppendLine("種　　別：" + state.Class);
            sb.AppendLine("行　　先：" + state.BoundFor);
            label2.Text = sb.ToString();
        }

        void SetNotch(int notch)
        {
            //ノッチ情報の送信
            TrainCrewInput.SetNotch(notch);


            if (notch == -8) label7.Text = "EB";
            else if (notch == -1) label7.Text = "抑速";
            else if (notch == 0) label7.Text = "N";
            else if (notch > 0) label7.Text = "P" + notch;
            else label7.Text = "B" + (-notch - 1);
        }

        private void comboBox2_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox cb = sender as ComboBox;
            InputAction tgtAction = InputAction.HornEle;
            switch (cb.Text)
            {
                case "勾配起動": tgtAction = InputAction.GradientStart; break;
                case "ブザー": tgtAction = InputAction.Buzzer; break;
                case "空笛": tgtAction = InputAction.HornAir; break;
                case "電笛": tgtAction = InputAction.HornEle; break;
                case "視点変更": tgtAction = InputAction.ViewChange; break;
                case "ポーズメニュー": tgtAction = InputAction.PauseMenu; break;
                case "ダイヤの表示・非表示": tgtAction = InputAction.ViewDiagram; break;
                case "UIの表示・非表示": tgtAction = InputAction.ViewUserInterface; break;
                case "視点のリセット": tgtAction = InputAction.ViewHome; break;
                case "EBリセット": tgtAction = InputAction.EBReset; break;
                case "右側小窓視点へ": tgtAction = InputAction.DriverViewR; break;
                case "左側小窓視点へ": tgtAction = InputAction.DriverViewL; break;
                case "運転席視点へ": tgtAction = InputAction.DriverViewC; break;
            }

            switch ((string)cb.Tag)
            {
                case "W": W_Action = tgtAction; break;
                case "G": G_Action = tgtAction; break;
                case "Y": Y_Action = tgtAction; break;
                case "R": R_Action = tgtAction; break;
            }
        }

    }
}
